\ ===================================================================
\ File: ASCII-FLOAT.FTH
\
\  This file provides words for saving floating point numeric data
\  to an ASCII file.
\
\   VARIABLE OUTFILE-ID ( -- a )   Holds the handle of the file.
\
\   : R>FILE ( r -- )   Takes a floating point number on the numeric
\      stack and writes it in ASCII format to the file represented by
\      the handle in OUTFILE-ID.  Follows the string with a delimiter.
\
\   : CR>FILE ( -- )   Writes a carriage return to the output file.
\
\   : SPACE-DELIMITED ( -- )   Sets the value delimiter to a space.
\
\   : TAB-DELIMITED ( -- )   Sets the value delimiter to a tab character.
\
\   : COMMA-DELIMITED ( -- )   Sets the value delimiter to a comma.
\
\
\  A typical word to save data is:
\  : SAVE-DATA ( -- )   "" FILE.NAME" COUNT r/w CREATE-FILE DROP OUTFILE-ID !
\     <code to save data using R>FILE and CR>FILE>
\     OUTFILE-ID @ CLOSE-FILE DROP ;
\
\              Copyright (C) 2014 Richard C. Wagner
\  This file is offered under the GNU General Public License.
\  Please see the license statement at the end of the file.
\ ===================================================================

\ The output file handle.
VARIABLE OUTFILE-ID

\ A buffer to build the ASCII string.
50 BUFFER: NUMBUF

\ The ASCII value delimeter.
0 VALUE DELIMITER

: SPACE-DELIMITED ( -- )   32 TO DELIMITER ;
: TAB-DELIMITED ( -- )   9 TO DELIMITER ;
: COMMA-DELIMITED ( -- )   44 TO DELIMITER ;


\ The carriage-return string.
CREATE $CR ( -- a )   2 C, 13 C, 10 C,

\ Number of decimal places in the string.
6 VALUE DECIMALS

\ Creates the ASCII representation of a float.
: RFORMAT ( r -- )
   DECIMALS (F.) DUP >R NUMBUF 1+ SWAP CMOVE R> NUMBUF C! ;

\ Output a floating point number to the current output file.
: R>FILE ( r -- )   RFORMAT  DELIMITER NUMBUF ADDCHAR
   NUMBUF COUNT OUTFILE-ID @ WRITE-FILE DROP ;

\ Output a carriage return to the current output file.
: CR>FILE ( -- )   $CR COUNT OUTFILE-ID @ WRITE-FILE DROP ;


\ ====================  License Statement  ==========================
\ This program is free software: you can redistribute it and/or modify
\ it under the terms of the GNU General Public License as published by
\ the Free Software Foundation, either version 3 of the License, or
\ (at your option) any later version.
\
\ This program is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
\ GNU General Public License for more details.
\
\ You should have received a copy of the GNU General Public License
\ along with this program.  If not, see <http://www.gnu.org/licenses/>.
\ ===================================================================